!*************************************************************************
! This file is part of
!
!       HiggsBounds 2.1.0 (fotran 77 version)
!
! by Philip Bechtle, Oliver Brein, Sven Heinemyer, Georg Weiglein
!    and Karina E. Williams.
!
!  Journal Reference: e-Print: arXiv:0811.4169 [hep-ph], accepted by CPC.
!  Web site: http://www.ippp.dur.ac.uk/HiggsBounds
!
! 2/2/2011
!*************************************************************************


#include "tables-nhmax.F"
#include "LEP-bound.F"
#include "TEV-bound.F"


*************************************************************************
	subroutine HiggsBounds_neutral_input_effC(Mh_val,GammaTot_h, 
     & g2hjss_s,g2hjss_p,g2hjcc_s,g2hjcc_p,g2hjbb_s,g2hjbb_p,
     & g2hjtt_s,g2hjtt_p,g2hjtautau_s,g2hjtautau_p,
     & g2hjWW,g2hjZZ,g2hjZga,g2hjgaga,g2hjgg,
     & g2hjhiZ, BR_hjinvisible_val,BR_hjhihi_val)
*************************************************************************
	implicit none
! input common blocks
#include "input_cb.h"

! input:
	double precision Mh_val(NHIGGS),GammaTot_h(NHIGGS)
	double precision 
     &          g2hjss_s(NHIGGS),g2hjss_p(NHIGGS),
     &          g2hjcc_s(NHIGGS),g2hjcc_p(NHIGGS),
     &		g2hjbb_s(NHIGGS),g2hjbb_p(NHIGGS),
     &		g2hjtt_s(NHIGGS),g2hjtt_p(NHIGGS),
     &		g2hjtautau_s(NHIGGS),g2hjtautau_p(NHIGGS),
     &		g2hjWW(NHIGGS),g2hjZZ(NHIGGS),g2hjZga(NHIGGS),g2hjgaga(NHIGGS),
     &		g2hjgg(NHIGGS),g2hjhiZ(NHIGGS,NHIGGS),
     &		BR_hjinvisible_val(NHIGGS),BR_hjhihi_val(NHIGGS,NHIGGS)

	double precision g2hjhiZ_compact((NHIGGS*(NHIGGS+1))/2)

! internal:
	integer i,j,k
	double precision sum_g2p,sum_g2s,vsmall,max_g2p,max_g2s
        double precision CS_lep_hjZ_ratio_val(NHIGGS),
     &		CS_lep_bbhj_ratio_val(NHIGGS),CS_lep_tautauhj_ratio_val(NHIGGS),
     &		CS_lep_hjhi_ratio_compact_val((NHIGGS*(NHIGGS+1))/2)

!  Tevatron CSs
	double precision CS_tev_gg_hj_ratio(NHIGGS),CS_tev_bb_hj_ratio(NHIGGS),
     & 		CS_tev_bg_hjb_ratio(NHIGGS),
     & 		CS_tev_ud_hjWp_ratio(NHIGGS),CS_tev_cs_hjWp_ratio(NHIGGS),
     & 		CS_tev_ud_hjWm_ratio(NHIGGS),CS_tev_cs_hjWm_ratio(NHIGGS),
     & 		CS_tev_dd_hjZ_ratio(NHIGGS),CS_tev_uu_hjZ_ratio(NHIGGS),
     & 		CS_tev_ss_hjZ_ratio(NHIGGS),CS_tev_cc_hjZ_ratio(NHIGGS),
     & 		CS_tev_bb_hjZ_ratio(NHIGGS)

	double precision CS_tev_pp_hj_ratio_val(NHIGGS), 
     &		CS_tev_pp_hjb_ratio_val(NHIGGS), 
     &		CS_tev_pp_hjW_ratio_val(NHIGGS), 
     &		CS_tev_pp_hjZ_ratio_val(NHIGGS),
     &		CS_tev_pp_vbf_ratio_val(NHIGGS),
     &		CS_tev_pp_tthj_ratio_val(NHIGGS)

! BRs
	double precision BR_hjss_val(NHIGGS),BR_hjcc_val(NHIGGS),
     &		BR_hjbb_val(NHIGGS),BR_hjtautau_val(NHIGGS),
     &		BR_hjWW_val(NHIGGS),BR_hjZZ_val(NHIGGS),BR_hjZga_val(NHIGGS),
     &		BR_hjgaga_val(NHIGGS),BR_hjgg_val(NHIGGS)

c>> check
c	write(*,*)'after:'
c	do i=1,NHIGGS
c	write(*,*)'g2hjhiZ(',i,',j)=',(g2hjhiZ(i,j),j=1,NHIGGS)
c	enddo
c        do i=1,NHIGGS
c	write(*,*)'BR_hjhihi_val(',i,',j)=',(BR_hjhihi_val(i,j),j=1,NHIGGS)
c	enddo
c<<


	do i=1,NHIGGS
	  Mh(i)=Mh_val(i)
	  BR_hjinvisible(i)=BR_hjinvisible_val(i)
	  do j=1,NHIGGS
	    BR_hjhihi(i,j)=BR_hjhihi_val(i,j)
	  enddo
	enddo

! translate g2hjhiZ to g2hjhiZ_compact
        k=0
        do j=1,NHIGGS
          do i=1,NHIGGS
            if(i .le. j) then
              k=k+1
              g2hjhiZ_compact(k)=g2hjhiZ(j,i)
            endif
          enddo
        enddo

! determine CP-properties of neutral Higgs bosons

	vsmall = 1d-16	

	do i=1,NHIGGS
	  max_g2p=max(g2hjss_p(i),g2hjcc_p(i),g2hjbb_p(i),g2hjtt_p(i),g2hjtautau_p(i))
	  max_g2s=max(g2hjss_s(i),g2hjcc_s(i),g2hjbb_s(i),g2hjtt_s(i),g2hjtautau_s(i))

	  if(max_g2p .lt. vsmall ) then
	    CP_value(i)=1
	  elseif(max_g2s .lt. vsmall) then
	    CP_value(i)=-1
	  else
	    CP_value(i)=0
	  endif
	enddo

	  call calc_neutral_CS_and_BR_from_couplings(NHIGGS,Mh_val,GammaTot_h,
     &		g2hjss_s,g2hjss_p,
     &		g2hjcc_s,g2hjcc_p,
     &		g2hjbb_s,g2hjbb_p,
     &		g2hjtt_s,g2hjtt_p,
     &		g2hjtautau_s,g2hjtautau_p,
     &		g2hjWW,g2hjZZ,g2hjZga,g2hjgaga,
     &		g2hjgg,g2hjhiZ_compact,
     &		CS_lep_hjZ_ratio_val,
     &		CS_lep_bbhj_ratio_val,CS_lep_tautauhj_ratio_val,
     &		CS_lep_hjhi_ratio_compact_val,
     &		CS_tev_gg_hj_ratio,CS_tev_bb_hj_ratio,CS_tev_bg_hjb_ratio,
     &		CS_tev_ud_hjWp_ratio,CS_tev_cs_hjWp_ratio,
     &		CS_tev_ud_hjWm_ratio,CS_tev_cs_hjWm_ratio,
     &          CS_tev_dd_hjZ_ratio,CS_tev_uu_hjZ_ratio,CS_tev_ss_hjZ_ratio,
     &		CS_tev_cc_hjZ_ratio,CS_tev_bb_hjZ_ratio,
     &          CS_tev_pp_vbf_ratio_val,CS_tev_pp_tthj_ratio_val,
     &		BR_hjss_val,BR_hjcc_val,BR_hjbb_val,BR_hjtautau_val,
     &		BR_hjWW_val,BR_hjZZ_val,BR_hjZga_val,BR_hjgaga_val,BR_hjgg_val)

	call calc_hadCS_from_partCS(NHIGGS, Mh_val,
     &          CS_tev_gg_hj_ratio, CS_tev_bb_hj_ratio, CS_tev_bg_hjb_ratio,
     &		CS_tev_ud_hjWp_ratio, CS_tev_cs_hjWp_ratio, 
     &		CS_tev_ud_hjWm_ratio, CS_tev_cs_hjWm_ratio,
     &          CS_tev_dd_hjZ_ratio, CS_tev_uu_hjZ_ratio, CS_tev_ss_hjZ_ratio, 
     &		CS_tev_cc_hjZ_ratio, CS_tev_bb_hjZ_ratio,
     &          CS_tev_pp_hj_ratio_val, CS_tev_pp_hjb_ratio_val, 
     &		CS_tev_pp_hjW_ratio_val, CS_tev_pp_hjZ_ratio_val)

! filling the common blocks
	do i=1,NHIGGS
	  Mh(i)=Mh_val(i)

	  BR_hjss(i)=BR_hjss_val(i)
	  BR_hjcc(i)=BR_hjcc_val(i)
	  BR_hjbb(i)=BR_hjbb_val(i)
	  BR_hjtautau(i)=BR_hjtautau_val(i)
	  BR_hjWW(i)=BR_hjWW_val(i)
	  BR_hjZZ(i)=BR_hjZZ_val(i)
	  BR_hjZga(i)=BR_hjZga_val(i)
	  BR_hjgaga(i)=BR_hjgaga_val(i)
	  BR_hjgg(i)=BR_hjgg_val(i)
	  BR_hjinvisible(i)=BR_hjinvisible_val(i)
	  do j=1,NHIGGS
	    BR_hjhihi(i,j)=BR_hjhihi_val(i,j)
	  enddo

 	  CS_tev_pp_hj_ratio(i)=CS_tev_pp_hj_ratio_val(i)
	  CS_tev_pp_hjb_ratio(i)=CS_tev_pp_hjb_ratio_val(i)
	  CS_tev_pp_hjW_ratio(i)=CS_tev_pp_hjW_ratio_val(i)
	  CS_tev_pp_hjZ_ratio(i)=CS_tev_pp_hjZ_ratio_val(i)
	  CS_tev_pp_vbf_ratio(i)=CS_tev_pp_vbf_ratio_val(i)
	  CS_tev_pp_tthj_ratio(i)=CS_tev_pp_tthj_ratio_val(i)

	  CS_lep_hjZ_ratio(i)=CS_lep_hjZ_ratio_val(i)
	  CS_lep_bbhj_ratio(i)=CS_lep_bbhj_ratio_val(i)
	  CS_lep_tautauhj_ratio(i)=CS_lep_tautauhj_ratio_val(i)
#ifdef DEBUG
	write(*,*)'Mh(',i,')=',Mh(i)
	write(*,*)'CP_value(',i,')=',CP_value(i)
	write(*,*)'BR_hjss(',i,')=',BR_hjss(i)
	write(*,*)'BR_hjcc(',i,')=',BR_hjcc(i)
	write(*,*)'BR_hjbb(',i,')=',BR_hjbb(i)
	write(*,*)'BR_hjtautau(',i,')=',BR_hjtautau(i)
	write(*,*)'BR_hjWW(',i,')=',BR_hjWW(i)
	write(*,*)'BR_hjZZ(',i,')=',BR_hjZZ(i)
	write(*,*)'BR_hjZga(',i,')=',BR_hjZga(i)
	write(*,*)'BR_hjgaga(',i,')=',BR_hjgaga(i)
	write(*,*)'BR_hjgg(',i,')=',BR_hjgg(i)
	write(*,*)'BR_hjinvisible(',i,')=',BR_hjinvisible(i)
	do j=1,NHIGGS
	write(*,*)'BR_hjhihi(',i,',',j,')=',BR_hjhihi(i,j)
	enddo
#endif
	enddo

!    translate CS_lep_hjhi_ratio_compact to CS_lep_hjhi_ratio
        k=0
        do j=1,NHIGGS
          do i=1,NHIGGS
            if(i .le. j) then
              k=k+1
              CS_lep_hjhi_ratio(j,i)=CS_lep_hjhi_ratio_compact_val(k)
              if(i .ne. j) CS_lep_hjhi_ratio(i,j)=CS_lep_hjhi_ratio(j,i)
            endif
          enddo
        enddo

	end

*************************************************************************
        subroutine HiggsBounds_neutral_input_part(Mh_val, GammaTot_h,
     &		CP_value_val,
     &		CS_lep_hjZ_ratio_val,
     &		CS_lep_bbhj_ratio_val,
     &		CS_lep_tautauhj_ratio_val,
     &		CS_lep_hjhi_ratio_val,
     &		CS_tev_gg_hj_ratio,
     &		CS_tev_bb_hj_ratio, 
     &		CS_tev_bg_hjb_ratio,
     &		CS_tev_ud_hjWp_ratio, CS_tev_cs_hjWp_ratio,
     &		CS_tev_ud_hjWm_ratio, CS_tev_cs_hjWm_ratio,
     &          CS_tev_dd_hjZ_ratio, CS_tev_uu_hjZ_ratio,
     &		CS_tev_ss_hjZ_ratio, CS_tev_cc_hjZ_ratio,
     &		CS_tev_bb_hjZ_ratio,
     &          CS_tev_pp_vbf_ratio_val,
     &		CS_tev_pp_tthj_ratio_val,
     &		BR_hjss_val, BR_hjcc_val, BR_hjbb_val, BR_hjtautau_val,
     &		BR_hjWW_val, BR_hjZZ_val, BR_hjZga_val, BR_hjgaga_val, BR_hjgg_val,
     & 		BR_hjinvisible_val, BR_hjhihi_val)
*************************************************************************
! remarks:	- GammaTot_h not yet needed for anything in this subroutine
!		  (probably will be in future versions).
	implicit none
! input common blocks
#include "input_cb.h"

! input:
        integer CP_value_val(NHIGGS)

	double precision Mh_val(NHIGGS), GammaTot_h(NHIGGS),
     &		CS_lep_hjZ_ratio_val(NHIGGS),
     &		CS_lep_bbhj_ratio_val(NHIGGS),
     &		CS_lep_tautauhj_ratio_val(NHIGGS),
     &		CS_lep_hjhi_ratio_val(NHIGGS,NHIGGS),
     &		CS_tev_gg_hj_ratio(NHIGGS),
     &		CS_tev_bb_hj_ratio(NHIGGS), 
     &		CS_tev_bg_hjb_ratio(NHIGGS),
     &		CS_tev_ud_hjWp_ratio(NHIGGS), 
     &		CS_tev_cs_hjWp_ratio(NHIGGS),
     &		CS_tev_ud_hjWm_ratio(NHIGGS), 
     &		CS_tev_cs_hjWm_ratio(NHIGGS),
     &          CS_tev_dd_hjZ_ratio(NHIGGS), 
     &		CS_tev_uu_hjZ_ratio(NHIGGS),
     &		CS_tev_ss_hjZ_ratio(NHIGGS), 
     &		CS_tev_cc_hjZ_ratio(NHIGGS),
     &		CS_tev_bb_hjZ_ratio(NHIGGS),
     &          CS_tev_pp_vbf_ratio_val(NHIGGS),
     &		CS_tev_pp_tthj_ratio_val(NHIGGS),
     &		BR_hjss_val(NHIGGS), BR_hjcc_val(NHIGGS), 
     &		BR_hjbb_val(NHIGGS), BR_hjtautau_val(NHIGGS),
     &		BR_hjWW_val(NHIGGS), BR_hjZZ_val(NHIGGS), BR_hjZga_val(NHIGGS), 
     &		BR_hjgaga_val(NHIGGS), BR_hjgg_val(NHIGGS),
     & 		BR_hjinvisible_val(NHIGGS), BR_hjhihi_val(NHIGGS,NHIGGS)

! internal:
	integer i,j
	double precision CS_tev_pp_hj_ratio_val(NHIGGS),
     &		CS_tev_pp_hjW_ratio_val(NHIGGS),
     &		CS_tev_pp_hjZ_ratio_val(NHIGGS),
     &		CS_tev_pp_hjb_ratio_val(NHIGGS)



	call calc_hadCS_from_partCS(NHIGGS, Mh_val,
     &          CS_tev_gg_hj_ratio, CS_tev_bb_hj_ratio, CS_tev_bg_hjb_ratio,
     &		CS_tev_ud_hjWp_ratio, CS_tev_cs_hjWp_ratio, 
     &		CS_tev_ud_hjWm_ratio, CS_tev_cs_hjWm_ratio,
     &          CS_tev_dd_hjZ_ratio, CS_tev_uu_hjZ_ratio, CS_tev_ss_hjZ_ratio, 
     &		CS_tev_cc_hjZ_ratio, CS_tev_bb_hjZ_ratio,
     &          CS_tev_pp_hj_ratio_val, CS_tev_pp_hjb_ratio_val, 
     &		CS_tev_pp_hjW_ratio_val, CS_tev_pp_hjZ_ratio_val)

! filling the common blocks
	do i=1,NHIGGS
	  Mh(i)=Mh_val(i)
	  CP_value(i)=CP_value_val(i)

	  BR_hjss(i)=BR_hjss_val(i)
	  BR_hjcc(i)=BR_hjcc_val(i)
	  BR_hjbb(i)=BR_hjbb_val(i)
	  BR_hjtautau(i)=BR_hjtautau_val(i)
	  BR_hjWW(i)=BR_hjWW_val(i)
	  BR_hjZZ(i)=BR_hjZZ_val(i)
	  BR_hjZga(i)=BR_hjZga_val(i)
	  BR_hjgaga(i)=BR_hjgaga_val(i)
	  BR_hjgg(i)=BR_hjgg_val(i)
	  BR_hjinvisible(i)=BR_hjinvisible_val(i)
	  do j=1,NHIGGS
	    BR_hjhihi(i,j)=BR_hjhihi_val(i,j)
	  enddo

 	  CS_tev_pp_hj_ratio(i)=CS_tev_pp_hj_ratio_val(i)
	  CS_tev_pp_hjb_ratio(i)=CS_tev_pp_hjb_ratio_val(i)
	  CS_tev_pp_hjW_ratio(i)=CS_tev_pp_hjW_ratio_val(i)
	  CS_tev_pp_hjZ_ratio(i)=CS_tev_pp_hjZ_ratio_val(i)
	  CS_tev_pp_vbf_ratio(i)=CS_tev_pp_vbf_ratio_val(i)
	  CS_tev_pp_tthj_ratio(i)=CS_tev_pp_tthj_ratio_val(i)

	  CS_lep_hjZ_ratio(i)=CS_lep_hjZ_ratio_val(i)
	  CS_lep_bbhj_ratio(i)=CS_lep_bbhj_ratio_val(i)
	  CS_lep_tautauhj_ratio(i)=CS_lep_tautauhj_ratio_val(i)
	  do j=1,NHIGGS
	    CS_lep_hjhi_ratio(i,j)=CS_lep_hjhi_ratio_val(i,j)
	  enddo
#ifdef DEBUG
	write(*,*)'Mh(',i,')=',Mh(i)
	write(*,*)'CP_value(',i,')=',CP_value(i)
	write(*,*)'BR_hjss(',i,')=',BR_hjss(i)
	write(*,*)'BR_hjcc(',i,')=',BR_hjcc(i)
	write(*,*)'BR_hjbb(',i,')=',BR_hjbb(i)
	write(*,*)'BR_hjtautau(',i,')=',BR_hjtautau(i)
	write(*,*)'BR_hjWW(',i,')=',BR_hjWW(i)
	write(*,*)'BR_hjZZ(',i,')=',BR_hjZZ(i)
	write(*,*)'BR_hjZga(',i,')=',BR_hjZga(i)
	write(*,*)'BR_hjgaga(',i,')=',BR_hjgaga(i)
	write(*,*)'BR_hjgg(',i,')=',BR_hjgg(i)
	write(*,*)'BR_hjinvisible(',i,')=',BR_hjinvisible(i)
	do j=1,NHIGGS
	write(*,*)'BR_hjhihi(',i,',',j,')=',BR_hjhihi(i,j)
	enddo
c... more
#endif
	enddo

	end

*************************************************************************
        subroutine HiggsBounds_neutral_input_hadr(Mh_val, GammaTot_h,
     &		CP_value_val,
     &		CS_lep_hjZ_ratio_val,
     &		CS_lep_bbhj_ratio_val,
     &		CS_lep_tautauhj_ratio_val,
     &		CS_lep_hjhi_ratio_val,
     &		CS_tev_pp_hj_ratio_val, 
     &		CS_tev_pp_hjb_ratio_val,
     &          CS_tev_pp_hjW_ratio_val, 
     &		CS_tev_pp_hjZ_ratio_val,
     &          CS_tev_pp_vbf_ratio_val,
     &		CS_tev_pp_tthj_ratio_val,
     &		BR_hjss_val, BR_hjcc_val, BR_hjbb_val, BR_hjtautau_val,
     &		BR_hjWW_val, BR_hjZZ_val, BR_hjZga_val, BR_hjgaga_val, BR_hjgg_val,
     & 		BR_hjinvisible_val, BR_hjhihi_val)
*************************************************************************
! remarks:	- GammaTot_h not yet needed for anything in this subroutine
!		  (probably will be in future versions).
        implicit none
! input common blocks
#include "input_cb.h"

	integer CP_value_val(NHIGGS)

	double precision Mh_val(NHIGGS), GammaTot_h(NHIGGS),
     &          CS_lep_hjZ_ratio_val(NHIGGS),
     &          CS_lep_bbhj_ratio_val(NHIGGS),
     &          CS_lep_tautauhj_ratio_val(NHIGGS),
     &          CS_lep_hjhi_ratio_val(NHIGGS,NHIGGS),
     &          CS_tev_pp_hj_ratio_val(NHIGGS), 
     &          CS_tev_pp_hjb_ratio_val(NHIGGS),
     &          CS_tev_pp_hjW_ratio_val(NHIGGS), 
     &          CS_tev_pp_hjZ_ratio_val(NHIGGS),
     &          CS_tev_pp_vbf_ratio_val(NHIGGS),
     &          CS_tev_pp_tthj_ratio_val(NHIGGS),
     &          BR_hjss_val(NHIGGS), BR_hjcc_val(NHIGGS), 
     &		BR_hjbb_val(NHIGGS), BR_hjtautau_val(NHIGGS),
     &          BR_hjWW_val(NHIGGS),BR_hjZZ_val(NHIGGS), 
     &		BR_hjZga_val(NHIGGS), BR_hjgaga_val(NHIGGS), BR_hjgg_val(NHIGGS),
     &          BR_hjinvisible_val(NHIGGS), BR_hjhihi_val(NHIGGS,NHIGGS)

! internal:
	integer i,j

c>> check
c	write(*,*)'after:'
c	do i=1,NHIGGS
c	write(*,*)'CS_lep_hjhi_ratio_val(',i,',j)=',(CS_lep_hjhi_ratio_val(i,j),j=1,NHIGGS)
c	enddo
c        do i=1,NHIGGS
c	write(*,*)'BR_hjhihi_val(',i,',j)=',(BR_hjhihi_val(i,j),j=1,NHIGGS)
c	enddo
c<<


! filling the common blocks
	do i=1,NHIGGS
	  Mh(i)=Mh_val(i)
	  CP_value(i)=CP_value_val(i)

	  BR_hjss(i)=BR_hjss_val(i)
	  BR_hjcc(i)=BR_hjcc_val(i)
	  BR_hjbb(i)=BR_hjbb_val(i)
	  BR_hjtautau(i)=BR_hjtautau_val(i)
	  BR_hjWW(i)=BR_hjWW_val(i)
	  BR_hjZZ(i)=BR_hjZZ_val(i)
	  BR_hjZga(i)=BR_hjZga_val(i)
	  BR_hjgaga(i)=BR_hjgaga_val(i)
	  BR_hjgg(i)=BR_hjgg_val(i)
	  BR_hjinvisible(i)=BR_hjinvisible_val(i)
	  do j=1,NHIGGS
	    BR_hjhihi(i,j)=BR_hjhihi_val(i,j)
	  enddo

 	  CS_tev_pp_hj_ratio(i)=CS_tev_pp_hj_ratio_val(i)
	  CS_tev_pp_hjb_ratio(i)=CS_tev_pp_hjb_ratio_val(i)
	  CS_tev_pp_hjW_ratio(i)=CS_tev_pp_hjW_ratio_val(i)
	  CS_tev_pp_hjZ_ratio(i)=CS_tev_pp_hjZ_ratio_val(i)
	  CS_tev_pp_vbf_ratio(i)=CS_tev_pp_vbf_ratio_val(i)
	  CS_tev_pp_tthj_ratio(i)=CS_tev_pp_tthj_ratio_val(i)

	  CS_lep_hjZ_ratio(i)=CS_lep_hjZ_ratio_val(i)
	  CS_lep_bbhj_ratio(i)=CS_lep_bbhj_ratio_val(i)
	  CS_lep_tautauhj_ratio(i)=CS_lep_tautauhj_ratio_val(i)
	  do j=1,NHIGGS
	    CS_lep_hjhi_ratio(i,j)=CS_lep_hjhi_ratio_val(i,j)
	  enddo
#ifdef DEBUG
	write(*,*)'Mh(',i,')=',Mh(i)
	write(*,*)'CP_value(',i,')=',CP_value(i)
	write(*,*)'BR_hjss(',i,')=',BR_hjss(i)
	write(*,*)'BR_hjcc(',i,')=',BR_hjcc(i)
	write(*,*)'BR_hjbb(',i,')=',BR_hjbb(i)
	write(*,*)'BR_hjtautau(',i,')=',BR_hjtautau(i)
	write(*,*)'BR_hjWW(',i,')=',BR_hjWW(i)
	write(*,*)'BR_hjZZ(',i,')=',BR_hjZZ(i)
	write(*,*)'BR_hjZga(',i,')=',BR_hjZga(i)
	write(*,*)'BR_hjgaga(',i,')=',BR_hjgaga(i)
	write(*,*)'BR_hjgg(',i,')=',BR_hjgg(i)
	write(*,*)'BR_hjinvisible(',i,')=',BR_hjinvisible(i)
	do j=1,NHIGGS
	write(*,*)'BR_hjhihi(',i,',',j,')=',BR_hjhihi(i,j)
	enddo
c... more
#endif
	enddo

	
	end

*************************************************************************
        subroutine HiggsBounds_charged_input(MHplus_val,GammaTot_Hp,
     &		CS_lep_HpjHmj_ratio_val,BR_tWpb_val,
     &		BR_tHpjb_val,BR_Hpjcs_val,BR_Hpjcb_val,BR_Hpjtaunu_val)
*************************************************************************
	implicit none
! input common blocks
#include "input_cb.h"

!input:
	double precision MHplus_val(NHPLUS),GammaTot_Hp(NHPLUS),
     &          CS_lep_HpjHmj_ratio_val(NHPLUS),
     &		BR_tWpb_val,BR_tHpjb_val(NHPLUS),
     &		BR_Hpjcs_val(NHPLUS),BR_Hpjcb_val(NHPLUS),
     &		BR_Hpjtaunu_val(NHPLUS)

!internal
	integer i

	BR_tWpb =  BR_tWpb_val

	do i=1,NHPLUS
	  MHplus(i) = MHplus_val(i)
	  CS_lep_HpjHmj_ratio(i) = CS_lep_HpjHmj_ratio_val(i) 
	  BR_tHpjb(i) = BR_tHpjb_val(i)
	  BR_Hpjcs(i) = BR_Hpjcs_val(i)
     	  BR_Hpjcb(i) = BR_Hpjcb_val(i)
	  BR_Hpjtaunu(i) = BR_Hpjtaunu_val(i)
	enddo

#ifdef DEBUG
	do i =1,NHPLUS
	 write(*,*)'MHplus(',i,')=',MHplus(i)
	 write(*,*)'CS_lep_HpjHmj_ratio(',i,')=',CS_lep_HpjHmj_ratio(i)
	 write(*,*)'BR_tHpjb(',i,')=',BR_tHpjb(i)
	 write(*,*)'BR_tWpb=',BR_tWpb
	 write(*,*)'BR_Hpjcs(',i,')=',BR_Hpjcs(i)
	 write(*,*)'BR_Hpjcb(',i,')=',BR_Hpjcb(i)
	 write(*,*)'BR_Hpjtaunu(',i,')=',BR_Hpjtaunu(i)
	enddo
#endif
	end



*************************************************************************
	subroutine run_HiggsBounds(HBresult,chan,obsratio,ncombined ) 
*************************************************************************
	implicit none
#include "input_cb.h"
! output:
	integer HBresult,chan,ncombined
	double precision obsratio
! internal:
        double precision BR_hjhihi_compact(NHIGGS**2-NHIGGS)
        double precision CS_lep_hjhi_ratio_compact((NHIGGS*(NHIGGS+1))/2)

	logical useLEP,useTEV,useSingH	
	integer sanity_flag,i,j,k,output_chan

        logical lep_allowed,lep_final_h_comb(NHIGGS)
        integer lep_mlratio
        integer lep_result_flag,lep_num_H_combined,lep_sim_mass_sum(NHIGGS)
        double precision lep_obsratio,lep_predratio,lep_model_R_sig_BR

        logical tev_allowed,tev_final_h_comb(NHIGGS)
        integer tev_mlratio
        integer tev_result_flag,tev_num_H_combined
        double precision tev_obsratio,tev_predratio,tev_model_R_sig_BR

	logical allowed,H_combined(NHIGGS)
        integer mlratio,experiment_id
c experiment_id : 1 : LEP, 2 : Tevatron
        integer sim_mass_sum(NHIGGS)
        double precision predratio,model_R_sig_BR

	call determine_topology_flags(whichexpt,useTEV,useLEP,useSingH)

	call check_input_sanity(useLEP,useTEV,useSingH,sanity_flag)


        if(sanity_flag .eq. 1) then
!tag1
	  if(useLEP) then
c check LEP bounds
            call check_LEP_bound(lep_allowed,lep_result_flag,lep_mlratio,
     &		lep_obsratio,lep_predratio,lep_num_H_combined,
     &		lep_final_h_comb,lep_model_R_sig_BR)
	  endif


	  if(useTEV) then
c check Tevatron bounds
	    call check_TEV_bound(tev_allowed,tev_result_flag,tev_mlratio,
     &		tev_obsratio,tev_predratio,tev_num_H_combined,
     &		tev_final_h_comb,tev_model_R_sig_BR)
	  endif
	


	  if(useLEP) then
                experiment_id = 1
                allowed = lep_allowed
                mlratio = lep_mlratio
                obsratio = lep_obsratio
                predratio = lep_predratio
                HBresult = lep_result_flag
	 	ncombined = lep_num_H_combined
          endif
	  if(useTEV) then
                experiment_id = 2
                allowed = tev_allowed
                mlratio = tev_mlratio
                obsratio = tev_obsratio
		predratio = tev_predratio
                HBresult = tev_result_flag
		ncombined = tev_num_H_combined
	  endif

	  if(useLEP .and. useTEV) then
c  decide on which ratio to choose from LEP or TEV and set flag: experiment_id
#ifdef VERBOSE
	    write(*,*)'Overall channel with highest sensitivity:'
#endif
            if(lep_predratio .ge. tev_predratio) then
                experiment_id = 1
                allowed = lep_allowed
                mlratio = lep_mlratio
                obsratio = lep_obsratio
                predratio = lep_predratio
                HBresult = lep_result_flag
		ncombined = lep_num_H_combined
            else
                experiment_id = 2
                allowed = tev_allowed
                mlratio = tev_mlratio
                obsratio = tev_obsratio
                predratio = tev_predratio
                HBresult = tev_result_flag
		ncombined = tev_num_H_combined
            endif
#ifdef VERBOSE
	        call determine_output_channel_number(experiment_id,
     & 							mlratio,output_chan)
	        call write_output_channel(6,NHIGGS,NHPLUS,whichexpt,output_chan)
        	write(*,*)'   R_sig_BR/S95_predicted =',predratio
        	write(*,*)'   R_sig_BR/S95_observed  =',obsratio
#endif
	  endif

	endif
!tag1
	if(sanity_flag .eq. 0) then
		HBresult=-1
		experiment_id=0
		ncombined = 0
	endif

! If obsratio is zero, than output_chan is set to zero.
! Realization: set experiment_id=0
!	       then determine_output_channel_number will set 
!	       output_chan=0
	if(obsratio .eq. 0d0) then
	  experiment_id=0
	  ncombined=0
	endif



	call determine_output_channel_number(experiment_id,mlratio,chan)


	end




*********************************************************************
        subroutine initialize_HiggsBounds(NHIGGS_val,NHPLUS_val,whichexpt_val)
*********************************************************************
        implicit none
#include "input_cb.h"

        integer NHIGGS_val,NHPLUS_val
	character*5 whichexpt_val

	CMDLINE = .FALSE.

	NHIGGS=NHIGGS_val
	NHPLUS=NHPLUS_val
	whichexpt=whichexpt_val

	call write_HiggsBounds_info()

        call initialize_SM_decay_data()
        call initialize_TEV_tables()
        call initialize_LEP_tables()

	open(87,file='Key.dat')
	call write_table_key(87,NHIGGS_val,NHPLUS_val,whichexpt_val)
	close(87)

        end

***************************************************
        subroutine finish_HiggsBounds()
***************************************************
        implicit none


        end

*************************************************************************
        subroutine write_HiggsBounds_info()
*************************************************************************
        implicit none

	write(*,*)
	write(*,*)'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
	write(*,*)'~                                                        ~'
	write(*,*)'~          HiggsBounds 2.0.0 (fortran77 version)         ~'
	write(*,*)'~                                                        ~'
	write(*,*)'~     Philip Bechtle, Oliver Brein, Sven Heinemeyer,     ~'
	write(*,*)'~            Georg Weiglein, Karina E. Williams          ~'
	write(*,*)'~                                                        ~'
	write(*,*)'~ arXiv:0811.4169, http://www.ippp.dur.ac.uk/HiggsBounds ~'
	write(*,*)'~                                                        ~'
	write(*,*)'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
	write(*,*)
	write(*,*)'HiggsBounds collects together results from '
	write(*,*)
	write(*,*)'    * the LEP collaborations and LEP Higgs Working Group'
	write(*,*)'    * the CDF and D0 Collaborations'
	write(*,*)'    * the program HDECAY (arXiv:hep-ph/9704448)'
	write(*,*)'    * TeV4LHC Higgs Working Group report'
	write(*,*)'      (see arXiv:hep-ph/0612172 and ref. therein)'
	write(*,*)

	end




*****************************************************************************
	subroutine determine_topology_flags(whichexpt,
     &		useTEV,useLEP,useSingH)
*****************************************************************************
	implicit none 

	character*5 whichexpt

	integer lnblnk
        external lnblnk

	logical useTEV,useLEP,useSingH


	useSingH = .FALSE.

	if(whichexpt .eq. 'onlyT') then
		useTEV=.TRUE.
		useLEP=.FALSE.
	elseif(whichexpt .eq. 'onlyL') then
		useTEV=.FALSE.
                useLEP=.TRUE.
	elseif((whichexpt .eq. 'LandT') .or. (whichexpt .eq. 'onlyP')) then
                useTEV=.TRUE.
                useLEP=.TRUE.
	elseif(whichexpt .eq. 'singH') then
               	useTEV=.TRUE.
               	useLEP=.TRUE.	
	       	useSingH = .TRUE.
	endif
	
	end


*****************************************************************************
        subroutine check_input_sanity(useLEP,useTEV,useSingH,
     &			sanity_flag)
*****************************************************************************
* check_input_sanity ignores all BRs which are not relevant to the experiment.
	implicit none
#include "input_cb.h"
	logical useLEP,useTEV,useSingH
	integer sanity_flag,i,j
	double precision sumBRneut(NHIGGS),sumBRchar(NHPLUS),sumBRt,fuzziness

	fuzziness=0.001d0	

	sanity_flag=1

! neutral Higgs	bosons:

	do j=1,NHIGGS
	  sumBRneut(j)=0d0
	  if(Mh(j) .le. 0d0) then
	    if(CMDLINE) write(*,7788) j,lineno
	    if(.not. CMDLINE) write(*,7789) j
 7788	    FORMAT('HB WARNING: zero or negative mass of neutral Higgs #',I3,' in input line',I6)
 7789	    FORMAT('HB WARNING: zero or negative mass of neutral Higgs #',I3)
            sanity_flag=0
	  endif
	enddo

	do j=1,NHIGGS

	  sumBRneut(j)=BR_hjss(j)+BR_hjcc(j)+BR_hjbb(j)
     &          +BR_hjtautau(j)+BR_hjWW(j)+BR_hjZZ(j)+BR_hjgaga(j)+BR_hjZga(j)
     &		+BR_hjgg(j)+BR_hjinvisible(j)

	  do i=1,NHIGGS
            sumBRneut(j)=sumBRneut(j)+BR_hjhihi(j,i)
          enddo

          if(sumBRneut(j) .gt. (1d0+fuzziness)) then
	     if(CMDLINE) write(*,7790) sumBRneut(j),j,lineno
	     if(.not. CMDLINE) write(*,7791) sumBRneut(j),j
 7790	     FORMAT('HB WARNING: sum of BRs larger than 1 (',G16.6,') for neutral Higgs #',I3,' in input line',I6)
 7791	     FORMAT('HB WARNING: sum of BRs larger than 1 (',G16.6,') for neutral Higgs #',I3)
! 21.01.2009 currently only a warning is given
!	     sanity_flag=0
	  endif
	enddo


! charged Higgs	bosons:

	do j=1,NHPLUS
	  sumBRchar(j)=0d0
	  if(MHplus(j) .le. 0d0) then
	    if(CMDLINE) write(*,7795) j,lineno
	    if(.not. CMDLINE) write(*,7796) j
 7795	    FORMAT('HB WARNING: zero or negative mass of charged Higgs #',I3,' in input line',I6)
 7796	    FORMAT('HB WARNING: zero or negative mass of charged Higgs #',I3)
            sanity_flag=0
	  endif
	enddo

	sumBRt=BR_tWpb

	do j=1,NHPLUS

	  sumBRt=sumBRt+BR_tHpjb(j)
	  sumBRchar(j)=BR_Hpjcs(j)+BR_Hpjcb(j)+BR_Hpjtaunu(j)

          if(sumBRchar(j) .gt. (1d0+fuzziness)) then
	     if(CMDLINE) write(*,8890) sumBRchar(j),j,lineno
	     if(.not. CMDLINE) write(*,8891) sumBRchar(j),j
 8890	     FORMAT('HB WARNING: sum of BRs larger than 1 (',G16.6,') for charged Higgs #',I3,' in input line',I6)
 8891	     FORMAT('HB WARNING: sum of BRs larger than 1 (',G16.6,') for charged Higgs #',I3)
! 21.01.2009 currently only a warning is given
!	     sanity_flag=0
	  endif
	enddo

          if(sumBRt .gt. (1d0+fuzziness)) then
	     if(CMDLINE) write(*,8895) sumBRt,lineno
	     if(.not. CMDLINE) write(*,8896) sumBRt
 8895	     FORMAT('HB WARNING: sum of BRs larger than 1 (',G16.6,') for top quark in input line',I6)
 8896	     FORMAT('HB WARNING: sum of BRs larger than 1 (',G16.6,') for top quark')
! 21.01.2009 currently only a warning is given
!	     sanity_flag=0
	  endif


	end



*****************************************************************************
        subroutine determine_output_channel_number(experiment_id,exp_chan,
     &						   output_chan)
*****************************************************************************
        implicit none
#include "input_cb.h"
#include "tables-indexing-lep_cb.h"
#include "tables-indexing-tev_cb.h"
	integer experiment_id,exp_chan,output_chan,tev_offset

	if(experiment_id .eq. 0) then
		output_chan = 0
	else
	if(whichexpt .eq. 'onlyT') then
		tev_offset=0
		output_chan=exp_chan
	elseif(whichexpt .eq. 'onlyL') then
		output_chan=exp_chan
	elseif((whichexpt .eq. 'LandT') .or. (whichexpt .eq. 'onlyP')) then
		tev_offset= nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS
     &			   +nLEP_NDCHN*NHIGGS**2
		if(experiment_id .eq. 1) then
			output_chan=exp_chan
		elseif(experiment_id .eq. 2) then
			output_chan=exp_chan+tev_offset
		endif
	elseif(whichexpt .eq. 'singH') then
		tev_offset=nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS
		if(experiment_id .eq. 1) then
			output_chan=exp_chan
		elseif(experiment_id .eq. 2) then
			output_chan=exp_chan+tev_offset
		endif
	endif
	endif

	end


#include "analyses-descriptions.h"

